@@ -17,6 +17,10 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "strutl.h"
 
 #include "macro.h"
@@ -27,22 +31,37 @@
 #include <ctype.h>
 #include <string.h>
 
-char * str_dup(const char *str)
+char *str_dup(const char *str)
 {
-  return str ? strcpy (malloc(strlen(str) + 1), str) : NULL;
+    char *dup = NULL;
+
+    if (str) {
+        size_t size = strlen(str) + 1;
+        dup = malloc(size);
+        if (dup) {
+            memcpy(dup, str, size);
+        }
+    }
+    return dup;
 }
 
-char * str_printf(const char *fmt, ...)
+char *str_printf(const char *fmt, ...)
 {
     /* Guess we need no more than 100 bytes. */
-    int len;
     va_list ap;
-    int size = 100;
-    char *tmp, *str = NULL;
+    int     len;
+    int     size = 100;
+    char   *tmp, *str = NULL;
+
+    while (1) {
+
+        tmp = realloc(str, size);
+        if (tmp == NULL) {
+            X_FREE(str);
+            return NULL;
+        }
+        str = tmp;
 
-    str = malloc(size);
-    while (1) 
-    {
         /* Try to print in the allocated space. */
         va_start(ap, fmt);
         len = vsnprintf(str, size, fmt, ap);
@@ -58,12 +77,6 @@ char * str_printf(const char *fmt, ...)
             size = len+1; /* precisely what is needed */
         else           /* glibc 2.0 */
             size *= 2;  /* twice the old size */
-
-        tmp = realloc(str, size);
-        if (tmp == NULL) {
-            return str;
-        }
-        str = tmp;
     }
 }
 
